/* eslint-disable no-console */
import { IDictionary, Http, Metadata } from '@zeedhi/core';
import {
  Select, ModalService, Modal, IModal, AlertService,
} from '@zeedhi/common';
import { Calendar } from '../components/calendar/Calendar';

export class CalendarioController {
  protected unitySelected: number;

  protected subUnitySelected: number;

  protected sectorSelected: number;

  protected typeLocalSelected: number;

  protected typeTaskSelected: number;

  protected ChecklistAgends: Array<Object>;

  private filterModal!: Modal;

  private seeChecklist: boolean;

  private seeGerChecklist: boolean;

  private SubUnities: Array<IDictionary> = [];

  private colorByUnity: boolean;

  constructor() {
    this.ChecklistAgends = [];
    this.unitySelected = -1;
    this.subUnitySelected = -1;
    this.sectorSelected = -1;
    this.typeTaskSelected = -1;
    this.typeLocalSelected = -1;
    this.colorByUnity = true;
    // this.user = window.localStorage.getItem('UserLogado');
    const allowChecklist = window.localStorage.getItem('idpermissschecklist');
    const allowGerChecklist = window.localStorage.getItem('idpermisssvistoria');

    if (allowChecklist !== 'N') {
      this.seeChecklist = true;
    } else {
      this.seeChecklist = false;
    }

    if (allowGerChecklist !== 'N') {
      this.seeGerChecklist = true;
    } else {
      this.seeGerChecklist = false;
    }
  }

  public async getChecklists(taskUrl: string, vistoriaUrl: string) {
    let responseC = [];
    let responseG = [];
    if (this.seeChecklist) {
      const response: IDictionary = await Http.get(taskUrl);
      responseC = response.data.dataset.ChecklistAgends;
    }
    if (this.seeGerChecklist) {
      const response = await Http.get(vistoriaUrl);
      responseG = response.data.dataset.GerenciamentoChks;
    }
    // const checklistAgends: Array<IDictionary> = responseC.data.dataset.ChecklistAgends;
    // const gerChks: Array<IDictionary> = responseG.data.dataset.GerenciamentoChks;
    const finalArray = responseC.concat(responseG);
    return finalArray;
  }

  public unityChange() {
    const component: Select = Metadata.getInstance('selectUnity');
    const currentUnity = component.value;
    if (currentUnity === null) {
      this.unitySelected = -1;
    } else {
      this.unitySelected = currentUnity;
    }
  }

  public subUnityChange() {
    const component: Select = Metadata.getInstance('subUnitySelect');
    const currentSubUnity = component.value;
    if (currentSubUnity === null) {
      this.subUnitySelected = -1;
    } else {
      this.subUnitySelected = currentSubUnity;
    }
  }

  public sectorChange() {
    const component: Select = Metadata.getInstance('selectSector');
    const currentSector = component.value;
    if (currentSector === null) {
      this.sectorSelected = -1;
    } else {
      this.sectorSelected = currentSector;
    }
    // this.loadEventsChange();
  }

  public typeLocalChange() {
    const component: Select = Metadata.getInstance('selectTypeLocal');
    const currentTypeLocal = component.value;
    if (currentTypeLocal === null) {
      this.typeLocalSelected = -1;
    } else {
      this.typeLocalSelected = currentTypeLocal;
    }
  }

  public typeTaskChange() {
    const component: Select = Metadata.getInstance('selectTypeTask');
    const currentTypeTask = component.value;
    if (currentTypeTask === null) {
      this.typeTaskSelected = -1;
    } else {
      this.typeTaskSelected = currentTypeTask;
    }
  }

  public loadEventsChange() {
    this.unityChange();
    this.subUnityChange();
    this.sectorChange();
    this.typeLocalChange();
    this.typeTaskChange();
    let urlToRequestC = '/ChecklistAgends/Checklist?';
    let urlToRequestG = '/GerenciamentoChks/Checklist?';
    let propertyOrganizer = 'NRLOCALCLIENTE';
    if (this.unitySelected !== -1) {
      this.colorByUnity = false;
      if (this.subUnitySelected !== -1) {
        urlToRequestC += `NRLOCALCLIENTE=${this.subUnitySelected}&`;
        urlToRequestG += `NRLOCALCLIENTE=${this.subUnitySelected}&`;
      } else {
        // unity = this.unitySelected;
        this.SubUnities.forEach((subUnity) => {
          urlToRequestC += `NRLOCALCLIENTE[]=${subUnity.NRLOCALCLIENTE}&`;
          urlToRequestG += `NRLOCALCLIENTE[]=${subUnity.NRLOCALCLIENTE}&`;
        });
      }
    } else {
      this.colorByUnity = true;
    }
    if (this.sectorSelected !== -1) {
      urlToRequestC += `NRSETOR=${this.sectorSelected}&`;
      urlToRequestG += `NRSETOR=${this.sectorSelected}&`;
      propertyOrganizer = 'NRSETOR';
    }
    if (this.typeLocalSelected !== -1) {
      urlToRequestC += `NRTIPOLOCALCLI=${this.typeLocalSelected}&`;
      urlToRequestG += `NRTIPOLOCALCLI=${this.typeLocalSelected}&`;
    }
    if (this.typeTaskSelected !== -1) {
      const addString = `NRCHECKLIST=${this.typeTaskSelected}&`;
      urlToRequestG += addString;
      urlToRequestC += addString;
    }
    this.getChecklists(urlToRequestC, urlToRequestG)
      .then((finalArray) => {
        this.organizeByProperty(finalArray, propertyOrganizer)
          .then((cards: IDictionary) => {
            const calendar: Calendar = Metadata.getInstance('my-calendar');
            this.fillCards(calendar, cards)
              .then(() => {
                AlertService.show({
                  name: 'basic-alert',
                  text: 'Filtro aplicado com sucesso',
                  timeout: 5000,
                  color: 'success',
                  top: true,
                  left: false,
                });
              })
              .catch((err: any) => {
                AlertService.show({
                  name: 'basic-error',
                  text: err,
                  timeout: 5000,
                  color: 'error',
                  top: true,
                  left: false,
                });
              });
          })
          .catch((err) => {
            AlertService.show({
              name: 'basic-error',
              text: err,
              timeout: 5000,
              color: 'error',
              top: true,
              left: false,
            });
          });
      })
      .catch((err) => {
        AlertService.show({
          name: 'basic-error',
          text: err,
          timeout: 5000,
          color: 'error',
          top: true,
          left: false,
        });
      });
  }

  public fillCards(calendar: Calendar, cards: IDictionary) {
    return new Promise((resolve) => {
      calendar.cards = [];
      // const colors = ['black', 'red', 'orange', 'green', 'blue'];
      const fullyObject: Array<IDictionary> = [];
      Object.values(cards).forEach((card: Array<IDictionary>) => {
        card.map((item: IDictionary) => {
          const check: IDictionary = {
            name: item.NMCHECKLISTAGEND ? item.NMCHECKLISTAGEND : item.NMGERENCIAMENTOCHK,
            start: item.DTHORAAGEND,
            end: item.DTHRFINALAGEND,
            color: this.colorByUnity ? item.DSCORIDLOCAL : item.DSCORIDATIVIDADE,
            timed: false,
          };
          fullyObject.push(check);
          return check;
        });
      });
      calendar.cards = fullyObject;
      resolve(fullyObject);
    });
  }

  public loadEvents({ component }: { component: Calendar }) {
    this.getChecklists('/ChecklistAgends/Checklist', '/GerenciamentoChks/Checklist')
      .then((checklistAgends: Array<IDictionary>) => {
        this.ChecklistAgends = checklistAgends;
        this.organizeByProperty(checklistAgends, 'NRLOCALCLIENTE')
          .then((cards: IDictionary) => {
            this.fillCards(component, cards);
          });
      });
  }

  public async organizeByProperty(cards: Array<IDictionary>, property: string) {
    const filteredCards: IDictionary = {};
    cards.forEach((card) => {
      const key = card[property];
      if (!(key in filteredCards)) {
        filteredCards[key] = [];
      }
      filteredCards[key].push(card);
    });
    return filteredCards;
  }

  public getMainUnities({ component }: { component: Select }) {
    // Http.get('/Locals?parent=null')
    const params = { IDTIPOLOCALCLI: 1 };
    this.localsRequest(params)
      .then((response: IDictionary) => {
        const locals: Array<IDictionary> = response.data.dataset.Locals;
        const filteredLocal: Array<IDictionary> = locals.map((local: IDictionary) => {
          const loc = {
            name: local.NMLOCALCLIENTE,
            value: local.NRLOCALCLIENTE,
          };
          return loc;
        });
        component.datasource.updateData(filteredLocal);
        return filteredLocal;
      })
      .catch((err) => {
        throw err;
      });
  }

  protected async localsRequest(param: IDictionary) {
    let urlToRequest = '/Locals?';
    Object.keys(param).forEach((key: string) => {
      urlToRequest = `${urlToRequest}${key}=${param[key]}&`;
    });
    const response = await Http.get(urlToRequest);
    return response;
  }

  public getSubUnities({ component }:{ component: Select }) {
    const currentMainUnities = component.value;
    const subSelect: Select = Metadata.getInstance('subUnitySelect');
    if (currentMainUnities !== null) {
      const params = { NRLOCALCLIENTEPAI: currentMainUnities };
      this.localsRequest(params)
        .then((response) => {
          const subLocals = response.data.dataset.Locals;
          this.SubUnities = subLocals;
          const filteredLocal = subLocals.map((subLocal: IDictionary) => {
            const subLoc = {
              name: subLocal.NMLOCALCLIENTE,
              value: subLocal.NRLOCALCLIENTE,
            };
            return subLoc;
          });
          subSelect.datasource.updateData(filteredLocal);
        });
    } else {
      subSelect.datasource.updateData([]);
    }
  }

  public getSectors({ component }: { component: Select }) {
    Http.get('/Sectors')
      .then((response) => {
        const sectors: Array<IDictionary> = response.data.dataset.Sectors;
        const filteredSectors: Array<IDictionary> = sectors.map((sector: IDictionary) => {
          const sec = {
            name: sector.NMSETOR,
            value: sector.NRSETOR,
          };
          return sec;
        });
        component.datasource.updateData(filteredSectors);
        return filteredSectors;
      })
      .catch((err) => {
        throw err;
      });
  }

  public getTypeTasks({ component }: { component: Select }) {
    Http.get('/Checklists')
      .then((response) => {
        const types: Array<IDictionary> = response.data.dataset.Checklists;
        const filteredTypes: Array<IDictionary> = types.map((item: IDictionary) => {
          const tp = {
            name: item.NMCHECKLIST,
            value: item.NRCHECKLIST,
          };
          return tp;
        });
        component.datasource.updateData(filteredTypes);
        return filteredTypes;
      })
      .catch((err) => {
        throw err;
      });
  }

  public getLocalTypes({ component }: { component: Select }) {
    Http.get('/TipoLocalClis')
      .then((response) => {
        const typesLocal: Array<IDictionary> = response.data.dataset.TipoLocalClis;
        const filteredTypes: Array<IDictionary> = typesLocal.map((item: IDictionary) => {
          const tp = {
            name: item.NMTIPOLOCALCLI,
            value: item.NRTIPOLOCALCLI,
          };
          return tp;
        });
        component.datasource.updateData(filteredTypes);
        return filteredTypes;
      })
      .catch((err) => {
        throw err;
      });
  }

  public createModal() {
    const modal: IModal = {
      name: 'filterModal',
      persistent: true,
      fullscreen: false,
      style: {
        background: '#black',
      },
      children: [
        {
          name: 'closeButtonHeader',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'colTitleText',
              component: 'ZdCol',
              align: 'center',
              cssClass: 'zd-pa-0',
              children: [
                {
                  name: 'titleText',
                  component: 'ZdText',
                  text: 'Filtrar por: ',
                  cssClass: 'title-modal',
                },
              ],
            },
          ],
          rightSlot: [
            {
              name: 'closeModalButton',
              component: 'ZdModalCloseButton',
              small: true,
              modalName: 'filterModal',
            },
          ],
        },
        {
          name: 'row2',
          component: 'ZdRow',
          cssClass: 'spacing',
          children: [
            {
              name: 'col2-1',
              component: 'ZdCol',
              cols: 6,
              children: [
                {
                  name: 'selectUnity',
                  component: 'ZdSelect',
                  autocomplete: false,
                  label: 'Unidade',
                  dataText: 'name',
                  dataValue: 'value',
                  datasource: {
                    data: [],
                  },
                  events: {
                    onCreated: this.getMainUnities.bind(this),
                    change: this.getSubUnities.bind(this),
                  },
                },
              ],
            },
            {
              name: 'col1-extra',
              component: 'ZdCol',
              cols: 6,
              children: [
                {
                  name: 'subUnitySelect',
                  component: 'ZdSelect',
                  autocomplete: false,
                  label: 'Sub unidade',
                  dataText: 'name',
                  dataValue: 'value',
                  datasource: {
                    data: [],
                  },
                  events: {
                    change: this.subUnityChange.bind(this),
                  },
                },
              ],
            },
          ],
        },
        {
          name: 'row3',
          component: 'ZdRow',
          children: [
            {
              name: 'col2-2',
              component: 'ZdCol',
              cols: 16,
              children: [
                {
                  name: 'selectSector',
                  component: 'ZdSelect',
                  autocomplete: false,
                  label: 'Setor',
                  dataText: 'name',
                  dataValue: 'value',
                  datasource: {
                    data: [],
                  },
                  events: {
                    onCreated: this.getSectors.bind(this),
                  },
                },
              ],
            },
            {
              name: 'col2-4',
              component: 'ZdCol',
              cols: 12,
              children: [
                {
                  name: 'selectTypeLocal',
                  component: 'ZdSelect',
                  autocomplete: false,
                  label: 'Tipo de Local',
                  dataText: 'name',
                  dataValue: 'value',
                  datasource: {
                    type: 'simple',
                    items: [],
                  },
                  events: {
                    onCreated: this.getLocalTypes.bind(this),
                  },
                },
              ],
            },
          ],
        },
        {
          name: 'row4',
          component: 'ZdRow',
          children: [
            {
              name: 'col2-3',
              component: 'ZdCol',
              cols: 12,
              children: [
                {
                  name: 'selectTypeTask',
                  component: 'ZdSelect',
                  autocomplete: false,
                  label: 'Tipo de Tarefa',
                  dataText: 'name',
                  dataValue: 'value',
                  datasource: {
                    type: 'simple',
                    items: [],
                  },
                  events: {
                    onCreated: this.getTypeTasks.bind(this),
                  },
                },
              ],
            },
          ],
        },
        {
          name: 'col-divider',
          component: 'ZdCol',
          cssClass: 'zd-pa-0 spacing',
          children: [
            {
              name: 'divider',
              component: 'ZdDivider',
            },
          ],
        },
        {
          name: 'row6',
          component: 'ZdRow',
          cssClass: 'spacing',
          children: [
            {
              name: 'col6-1',
              component: 'ZdCol',
              cols: 12,
              align: 'right',
              children: [
                {
                  name: 'applyFilters',
                  component: 'ZdButton',
                  label: 'Aplicar',
                  width: 90,
                  outline: true,
                  events: {
                    click: this.loadEventsChange.bind(this),
                  },
                },
              ],
            },
          ],
        },
      ],
    };
    this.filterModal = ModalService.create(modal);
  }

  public destroyModal() {
    this.filterModal.destroy();
  }

  public openModal() {
    this.filterModal.show();
  }

  public closeModal() {
    this.filterModal.hide();
  }
}
